/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   lem_in.c                                           :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: mndhlovu <marvin@42.fr>                    +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2019/03/25 06:31:05 by mndhlovu          #+#    #+#             */
/*   Updated: 2019/04/15 17:55:43 by tmaze            ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "lem_in.h"

static t_node	*get_node_role(t_lmdata *data, char role)
{
	t_node	*it;

	it = data->nodes_data;
	while (it)
	{
		if (it->role == role)
			return (it);
		it = it->next;
	}
	return (NULL);
}

static void		get_nb_paths_max(t_lmdata *data, int start, int end)
{
	int		i;
	int		j;
	t_ind	*it;

	i = 0;
	j = 0;
	it = data->adj[start];
	while (it && ++i)
		it = it->next;
	it = data->adj[end];
	while (it && ++j)
		it= it->next;
	data->nb_paths_max = (i > j) ? j : i;
}

static int		lem_in(t_syntax *synt, t_holder *holder,
						t_lmdata *lmdata, t_path **ret)
{
	if (!(lm_parser(synt, lmdata, holder)))
		return (0);
	if (!(lm_verify_cmd(synt, holder, lmdata)))
		return (0);
	if (!lst_indinit(lmdata))
		return (0);
	if (!(lm_adj_parser(lmdata, holder)))
		return (0);
	lm_clear_unv(holder);
	get_nb_paths_max(lmdata, get_node_role(lmdata, 's')->ind
					, get_node_role(lmdata, 'e')->ind);
	if ((*ret = edmunds_karp(lmdata, get_node_role(lmdata, 's')->ind
							 , get_node_role(lmdata, 'e')->ind)) == NULL)
		return (0);
	if (!push_ants(lmdata, *ret))
		return (0);
	clean_path(*ret);
	return (1);
}

int				lm_error_exit(void)
{
	ft_putendl_fd("ERROR", 2);
	return (1);
}

int				main(int ac, char **av)
{
	t_syntax	synt;
	t_lmdata	ldata;
	t_holder	holder;
	t_path		*ret;

	(void)av;
	if (ac == 1)
	{
		lm_init_data(&synt, &holder, &ldata);
		if (!(lem_in(&synt, &holder, &ldata, &ret)))
			return (lm_error_exit());
		lm_clean_data(&ldata);
		return (0);
	}
	return (1);
}
